import Vue from 'vue'

const time = {
    toLocalDateTimeString(dateString) {
      try{
        if(typeof dateString !== 'string') {
          let date = new Date(dateString[0], (dateString[1] - 1), dateString[2], dateString[3], dateString[4], dateString[5]);
          date.setHours(date.getHours() + 9); 
          return date.toISOString().replace('T', ' ').substring(0, 19);
        } else {
          let date = new Date(dateString); 
          date.setHours(date.getHours() + 9); 
          return date.toISOString().replace('T', ' ').substring(0, 19);
        }
      }catch{
        return ''
      }
      
      
    },
    getTimeFromNow(date) {
        const now = new Date();
        const today = now
        const timeValue = new Date(date);

        const betweenTime = Math.floor((today.getTime() - timeValue.getTime()) / 1000 / 60);

        if (betweenTime < 1) return '방금전';
        if (betweenTime < 60) {
            return `${betweenTime}분전`;
        }

        const betweenTimeHour = Math.floor(betweenTime / 60);
        if (betweenTimeHour < 24) {
            return `${betweenTimeHour}시간전`;
        }

        const betweenTimeDay = Math.floor(betweenTime / 60 / 24);
        if (betweenTimeDay < 365) {
            return `${betweenTimeDay}일전`;
        }

        return `${Math.floor(betweenTimeDay / 365)}년전`;
    },

    getTimeFromAnotherTime(targetDate, baseDate) {
      const now = new Date(baseDate);
      const today = now
      const timeValue = new Date(targetDate);

      const betweenTime = Math.floor((today.getTime() - timeValue.getTime()) / 1000 / 60);

      if (betweenTime < 1) {
        return `${betweenTime * 60}초`;
      }
      if (betweenTime < 60) {
          return `${betweenTime}분`;
      }

      const betweenTimeHour = Math.floor(betweenTime / 60);
      if (betweenTimeHour < 24) {
          return `${betweenTimeHour}시간`;
      }

      const betweenTimeDay = Math.floor(betweenTime / 60 / 24);
      if (betweenTimeDay < 365) {
          return `${betweenTimeDay}일`;
      }

      return `${Math.floor(betweenTimeDay / 365)}년`;
  },

    getElapsedTimeFromSecond(second){
      let min = Math.floor(second/60);
      let hour = Math.floor(min/60);
      let sec = second%60;
      min = min%60;

      let th = hour;
      let tm = min;
      let ts = sec;

      if(th<10){
        th = "0" + hour;
      }
      if(tm < 10){
        tm = "0" + min;
      }
      if(ts < 10){
        ts = "0" + sec;
      }

      return  th + ":" + tm + ":" + ts
    },
    dateToFormatKorean (date) {
      if(typeof date !== 'string') {
        return date[0] + '년 ' + date[1] + '월 ' + date[2] +'일'
      } else {
        let dateString = date.substring(0,10)
        return dateString.substring(0,4) + '년 ' + dateString.substring(5,7) + '월 ' + dateString.substring(8) +'일'
      }
      
    },
    dateTimeToFormatKorean (date) {
      if(typeof date !== 'string') {
        date = this.convertDateArrayToString(date)
      }
      var week = ['일', '월', '화', '수', '목', '금', '토'];
      var dayOfWeek = week[new Date(date).getDay()];
      return date.substring(0,4) + '년 ' + date.substring(5,7) + '월 ' + date.substring(8,10) +'일 ' + dayOfWeek + '요일 ' + date.substring(11,19)
    },
    dateShotToFormatKorean (date) {
      if(typeof date !== 'string') {
        return date[0] + '.' + date[1] + '.' + date[2]
      } else {
        let dateString = date.substring(0,10)
        return dateString.substring(0,4) + '.' + dateString.substring(5,7) + '.' + dateString.substring(8)
      }
    },
    simpleFormat(date){
      let data = new Date(date)
      return data.toLocaleDateString();
    },
    convertDateArrayToString(dateArray) {
      try{
        let date = dateArray[0] + "-" + this.addTimeBlank(dateArray[1]) + "-" + this.addTimeBlank(dateArray[2]) + "T" + this.addTimeBlank(dateArray[3])+":" + this.addTimeBlank(dateArray[4]) + ":" + this.addTimeBlank(dateArray[5])
        this.addTimeBlank(dateArray[3])
        return date
      }catch{
        return new Date().toLocaleTimeString()
      }
    },
    addTimeBlank(time) {
      return time.toString().length === 1 ? '0' + time : time.toString()
    }
}

const utils = {
    // 소수점2자리까지의 바이트 단위로 변환시키는 함수
    convertByteToString(bytes, decimals = 2) {
        if (bytes === 0) return '0 Bytes';
        const k = 1024;
        const dm = decimals < 0 ? 0 : decimals;
        const sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
        const i = Math.floor(Math.log(bytes) / Math.log(k));
        return parseFloat((bytes / Math.pow(k, i)).toFixed(dm)) + ' ' + sizes[i];
    },

    checkViewMode(level, vuetify) {
      let viewMode = ['xs', 'sm', 'md', 'lg', 'xl']
      if(viewMode.slice(0,level).indexOf(vuetify.breakpoint.name) !== -1) {
        return false
      } else {
        return true
      }
    },

    btoa(content) {
      return window.btoa(content)
    },

    btoaEscape(content) {
      return window.btoa(encodeURIComponent(content).replace(/%([0-9A-F]{2})/g,function(match,p1){return String.fromCharCode('0x'+p1)}))
    },

    // 이미지파일과 어노테이션 파일을 Zip으로 압축해서 다운받는 함수
    async downloadDatasetZip(files, name){
        let zip = new JSZip()

        return new Promise((resolve, reject) => {
            for (let i = 0; i < files.length; i++) {
                let filename = files[i].filename
                let folder = files[i].folder
                zip.file(folder+"/"+filename, files[i].image,{ base64: true, createFolders: true })
                filename = filename.substring(0,filename.indexOf("."))
                zip.file(folder+"/"+filename + '.xml', atob(files[i].annotation))
            }
            zip.generateAsync({
                type: "blob"
            }).then((content) => {
                FileSaver.saveAs(content, name + "_dataset_" + Date.now() + ".zip")
                resolve()
            })
        })
    },

    getSizeOfImageFile(file) {
      return new Promise((resolve, reject)=> {
        let img= new Image();
        let src = URL.createObjectURL(file);
        let size = {
          width:0,
          height:0
        }
        img.src = src
        img.onload = function () {
          size.width = this.width
          size.height = this.height
          resolve(size)
        };
        img.onerror = reject
      })
    },

    // 클립보드 저장 함수
    copy(val, callback = null) {
      if(val) {
        if(navigator.clipboard) {
            navigator.clipboard.writeText(val).then(() => {
              
            })
        } else {
          const t = document.createElement("textarea");
          document.body.appendChild(t);
          t.value = val;
          t.select();
  
          document.execCommand('copy');
          document.body.removeChild(t);
        }

        if(callback !== null)
          callback()
      }
    },

    isNumeric(num, opt){
      // 좌우 trim(공백제거)을 해준다.
      num = String(num).replace(/^\s+|\s+$/g, "");
     
      if(typeof opt == "undefined" || opt == "1"){
        // 모든 10진수 (부호 선택, 자릿수구분기호 선택, 소수점 선택)
        var regex = /^[+\-]?(([1-9][0-9]{0,2}(,[0-9]{3})*)|[0-9]+){1}(\.[0-9]+)?$/g;
      }else if(opt == "2"){
        // 부호 미사용, 자릿수구분기호 선택, 소수점 선택
        var regex = /^(([1-9][0-9]{0,2}(,[0-9]{3})*)|[0-9]+){1}(\.[0-9]+)?$/g;
      }else if(opt == "3"){
        // 부호 미사용, 자릿수구분기호 미사용, 소수점 선택
        var regex = /^[0-9]+(\.[0-9]+)?$/g;
      }else{
        // only 숫자만(부호 미사용, 자릿수구분기호 미사용, 소수점 미사용)
        var regex = /^[0-9]$/g;
      }
     
      if( regex.test(num) ){
        num = num.replace(/,/g, "");
        return isNaN(num) ? false : true;
      }else{ return false;  }
    }
}

const properties = {
  getImageCdnUrl(key, w, h, q) {
    let f = key.split('.').pop() === 'png' ? 'webp' : ''
    let url = 'https://d2q9yzkd471o7j.cloudfront.net/' + key + '?w=' + w + '&h=' + h + '&f=' + f + '&q=' + q 
    return url
  },
  getAvatar(key, w, h, q) {
    let defaultAvatar = []
    
    for(let i=1; i<=8; ++i){
      for(let j=1; j<=3; ++j){
        defaultAvatar.push(`/profileAvatar/${i}-${j}.png`)
      }
    }
    
    for(let i=1; i<=6; ++i){
      defaultAvatar.push(`/profileSmaple${i}.png`)
    }
    
    if(typeof key === 'undefined' || key === null) {
      return '/default-image.png'
    }else if(defaultAvatar.indexOf(key) !== -1) {
      return key
    }else{
      return this.getImageCdnUrl(key, w, h, q)
    }
  },
  getProjectKey(orgId, projectId, projectType) {
    return btoa(orgId + '/' + projectType.toLowerCase() + '/' + projectId)
  },

  // auth = $auth
  getUserInfo(auth) {
    let user = auth.user.body
    return {
      id: user.id,
      name: user.name,
      email: user.email,
      avatar: user.avatar
    }
  },

  getUserInfoBase64(auth) {
    return Buffer.from(JSON.stringify(this.getUserInfo(auth))).toString("base64")
  },

  // 공개용(데모용) 조직인지 확인
  isPublicOrg(orgId) {
    const publicOrgsId = ['6c34d2b00b604a689953cb0d5bd4b1cf','o-84e97db78ba14166ae523aa2283a378f']
    return orgId && publicOrgsId.indexOf(orgId) !== -1
  }
}

const regular = {
  basic(data){
    let reg=/[\{\}\[\]\/?.,;:|\)*~`!^\-_+<>@\#$%&\\\=\(\'\"]/gi;
    return reg.test(data)
  },
  noBlank(data){ // 공백 없을 경우 true 반환
    let reg = /[\s]/g;
    return !reg.test(data)
  },
  passwordPatten(data){
    //영문,숫자,특수문자(!@$%^&* 만 허용) 9 이상
    let reg= /^(?=.*[A-Za-z])(?=.*\d)(?=.*[$@$!%*#?&])[A-Za-z\d$@$!%*#?&]{9,20}$/g
    return reg.test(data)
  },
  emailPatten(data){
    let reg = /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/
    return reg.test(data)
  },
  name(data){
    let reg =  /^[ㄱ-ㅎ가-힣a-zA-Z\s]+$/
    return reg.test(data)
  },
  projName(data){ // 한글,영문,숫자,공백만 입력되었는지 TEST
    let reg =  /^[ㄱ-ㅎ가-힣a-zA-Z0-9\s]+$/
    return reg.test(data)
  },
  projDescription(data){ // 한글,영문,숫자,공백,특수문자만 입력되었는지 TEST
    let reg =  /^[ㄱ-ㅎ가-힣a-zA-Z0-9\s!?@#$%^&*():;+-=~{}<>\_\[\]\|\\\"\'\,\.\/\`\₩]+$/
    return reg.test(data)
  },
  number(data){
    let reg = /^[0-9]/
    return reg.test(data)
  },
  phone(data) {
    let reg = /^[+]*[(]{0,1}[0-9]{1,3}[)]{0,1}[-\s\./0-9]*$/g
    return reg.test(data)
  }
}

const rules = {
  title: [
    v => v.length > 0 || '값을 입력해주세요.',
    v => v.length <= 20 || '20자 초과되었습니다.',
    v => regular.noBlank(v[0]) || '공백으로 시작할 수 없습니다.',
    v => v === "" || regular.projName(v) || '한글,영문,숫자 외의 문자가 입력되었습니다.'
  ],
  description: [
    v => v.length <= 100 || '100자 초과되었습니다.',
    v => v === "" || regular.projDescription(v) || '적절하지 않은 문자가 입력되었습니다.'
  ],

}
  

const lang = {
  getString(key){
    let la = navigator.language || navigator.userLanguage
    if(la.indexOf('en')!==-1) {
      return  dictionary['en'][key]
    } else {
      return  dictionary['ko'][key]
    }
  }
}

const dictionary = {
  ko:{
    dashboard: '대시보드',
    alert_save:'성공적으로 저장되었습니다.',
  },
  en:{
    dashboard: 'dashboard',
    alert_save:'It has been saved successfully.',
  }
}

const move = {
  async setOrganizationAndProject(project, axios, store) {
    
    // 프로젝트 참여 여부 확인
    const isParticipated = await axios.get('/auth/project/user-list',{
      params:{
        userId: store.state.auth.user.body.id
      }
    })
    .then(res=>{
      const projectList = res.data.body
      if(projectList.findIndex(item => item.id === project.id) > -1){
        return true
      } else {
        return false
      }
    })
    .catch(err=>{
      console.log('프로젝트 참여 여부 확인 중 에러발생 err : ', err)
    })
    
    if(isParticipated){
      if(project.organizationId !== store.state.organizationId) {
        axios.defaults.headers.common['orgId'] = project.organizationId
        store.commit('setOrganizationId', project.organizationId)
        store.commit('setKeepOrganizationId', project.organizationId)
      }
      axios.defaults.headers.common['projectId'] = project.id
      store.commit('setProject', project)
    } else {
      alert("프로젝트 접근 권한이 없습니다.")
    }
    
    return isParticipated
  },
  async goOrg(org, axios, store, router) {
    store.state.organization = org
    axios.defaults.headers.common['orgId'] = org.orgId
    store.commit('setOrganizationId', org.orgId)
    store.commit('setKeepOrganizationId', org.orgId)
    router.push('/console')
  },

  async goProject(project, axios, store, router) {
    if(await this.setOrganizationAndProject(project,axios,store))
      router.push({path:'/console/project/'+ project.id})
  },

  async goProjectById(projectId, axios, store, router) {
    let project = null
    await axios.get('/auth/project/' + projectId, {
      params:{
        m: true
      }
    })
    .then(res=> {
      project= res.data.body
    })
    
    if(await this.setOrganizationAndProject(project,axios,store))
      router.push({path:'/console/project/'+ project.id})
  },

  async goProjectModel(project, model, axios, store, router) {
    if(await this.setOrganizationAndProject(project,axios,store))
      router.push({path:'/console/project/'+ project.id +'/models/' + model.versionId})
  },

  async goProjectWebPoint(project, point, axios, store, router) {
    if(await this.setOrganizationAndProject(project,axios,store))
      router.push({path:'/console/project/'+ project.id +'/webpoint/' + point.id})
  },

  async goProjectEdgePoint(project, point, axios, store, router) {
    if(await this.setOrganizationAndProject(project,axios,store) ){
      router.push({path:'/console/project/' + project.id + '/edges/' + point.id})
    }
  },

  async goProjectCallback(projectId, axios, store, callback) {
    let project = null
    await axios.get('/auth/project/' + projectId, {
      params:{
        m: true
      }
    })
    .then(res=> {
      project= res.data.body
    })
    
    if(await this.setOrganizationAndProject(project,axios,store)){
      if(callback) {
        callback()
      }
    }
  },
}

const colors = {
  colToHex(c) {
    let color = (c < 75) ? c + 75 : c
    let hex = color.toString(16);
    return hex.length == 1 ? "0" + hex : hex;
  },
  rgbToHex(r,g,b) {
    return "#" + this.colToHex(r) + this.colToHex(g) + this.colToHex(b);
  },
  getRandomColor() {
    return this.rgbToHex(
      Math.floor(Math.random() * 255),
      Math.floor(Math.random() * 255),
      Math.floor(Math.random() * 255));
  },
  getRandomColorByEmail(email) {
    return this.rgbToHex(
      Math.floor((email.length * email.length * email.charCodeAt(0)) % 255),
      Math.floor((email.length * email.length * email.charCodeAt(1)) % 255),
      Math.floor((email.length * email.length * email.charCodeAt(2)) % 255));
  },
  getSplitColor(i) {
    if(i === 1 || i === 'VALID') {
      return '#2E7D32'
    } else if (i === 2 || i === 'TEST') {
      return '#9E9D24'
    } else {
      return '#01579B'
    }
  },
}



Vue.prototype.$time = time;
Vue.prototype.$utils = utils;
Vue.prototype.$regular = regular;
Vue.prototype.$rules = rules;
Vue.prototype.$lang = lang;
Vue.prototype.$properties = properties;
Vue.prototype.$colors = colors;
Vue.prototype.$move = move;

export default ({app}, inject) => {
  inject('time', time);
  inject('utils', utils);
  inject('regular', regular);
  inject('rules', rules);
  inject('lang', lang);
  inject('properties', properties);
  inject('colors', colors);
  inject('move', move);
}
